<?php
/*--------------------------------------------------------------------------------------------------
    PropertyQuantityReadService.php 2020-07-08
    Gambio GmbH
    http://www.gambio.de
    Copyright (c) 2020 Gambio GmbH
    Released under the GNU General Public License (Version 2)
    [http://www.gnu.org/licenses/gpl-2.0.html]
 -------------------------------------------------------------------------------------------------*/

declare(strict_types=1);
namespace Gambio\Shop\Properties\SellingUnitQuantitiy\Database\Services;

use Gambio\Shop\ProductModifiers\Modifiers\Collections\ModifierIdentifierCollection;
use Gambio\Shop\ProductModifiers\Modifiers\Collections\ModifierIdentifierCollectionInterface;
use Gambio\Shop\Properties\Database\Services\Interfaces\PropertiesReaderServiceInterface;
use Gambio\Shop\Properties\ProductModifiers\Database\ValueObjects\PropertyModifierIdentifier;
use Gambio\Shop\Properties\Properties\Collections\CombinationCollectionInterface;
use Gambio\Shop\Properties\Properties\Entities\Combination;
use Gambio\Shop\Properties\SellingUnitQuantitiy\Quantitiy\Entities\AbstractPropertyQuantity;
use Gambio\Shop\Properties\SellingUnitQuantitiy\Quantitiy\Entities\CombinationQuantity;
use Gambio\Shop\Properties\SellingUnitQuantitiy\Quantitiy\Entities\PropertyTotalQuantity;
use Gambio\Shop\Properties\SellingUnitQuantitiy\Quantitiy\Exceptions\ProductDoesntUsePropertyQuantityException;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\Interfaces\QuantityInterface;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\Interfaces\ScopedQuantityInterface;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\SellingUnitId;
use ProductDataInterface;

class PropertyQuantityReadService implements PropertyQuantityReadServiceInterface
{
    /**
     * @var bool
     */
    protected $attributeStockCheck;
    /**
     * @var PropertiesReaderServiceInterface
     */
    protected $propertiesReaderService;
    /**
     * @var bool
     */
    protected $stockCheck;


    /**
     * PropertyQuantityReadService constructor.
     *
     * @param PropertiesReaderServiceInterface $propertiesReaderService
     * @param bool $stockCheck
     * @param bool $attributeStockCheck
     */
    public function __construct(
        PropertiesReaderServiceInterface $propertiesReaderService,
        bool $stockCheck,
        bool $attributeStockCheck
    ) {
        $this->propertiesReaderService = $propertiesReaderService;
        $this->stockCheck = $stockCheck;
        $this->attributeStockCheck = $attributeStockCheck;
    }


    /**
     * @inheritDoc
     */
    public function getQuantityBy(SellingUnitId $id, ProductDataInterface $product, QuantityInterface $requested): ?AbstractPropertyQuantity
    {
        if (($product->getPropertiesQuantityCheck() === 2)
            || ($product->getPropertiesQuantityCheck() === 0 && $this->stockCheck && $this->attributeStockCheck)) {

            $combinations = $this->propertiesReaderService->getCombinationsFor($id);

            if (count($combinations) === 1) {
                return $this->getQuantityByCombination($id, $combinations[0], $product, $requested);
            }

            if (count($combinations) > 0) {

                return $this->sumUpQuantities($id, $product, $combinations, $requested);
            }
        }

        return null;
    }

    /**
     * @inheritDoc
     */
    public function getQuantityByCombination(
        SellingUnitId $sellingUnitId,
        Combination $combination,
        ProductDataInterface $product,
        QuantityInterface $requested
    ): ?CombinationQuantity {
        try {

            $collection = $this->createModifierIdCollection($sellingUnitId);

            $qty = $combination->quantity()->value();

            if($requested && $requested instanceof ScopedQuantityInterface) {
                /** @var ScopedQuantityInterface $requested */
                $qty -= $requested->scope()->quantityFor($sellingUnitId->productId(), $collection);
            }
            return new CombinationQuantity($qty,
                $product->measureUnit(),
                $combination->id(),
                $this->stockCheck,
                $this->attributeStockCheck,
                $product->getPropertiesQuantityCheck(),
                $collection);
        } catch (ProductDoesntUsePropertyQuantityException $e) {
            return null;
        }
    }

    /**
     * @param SellingUnitId $sellingUnitId
     * @param ProductDataInterface $product
     * @param CombinationCollectionInterface $combinations
     * @param QuantityInterface|null $requested
     * @return PropertyTotalQuantity|null
     */
    public function sumUpQuantities(
        SellingUnitId $sellingUnitId,
        ProductDataInterface $product,
        CombinationCollectionInterface $combinations,
        ?QuantityInterface $requested = null
    ): ?PropertyTotalQuantity {
        if (count($combinations) === 0) {
            return null;
        }


        try {
            $qty = 0;
            foreach ($combinations as $combination) {
                $qty += $combination->quantity()->value();
            }
            $collection = $this->createModifierIdCollection($sellingUnitId);

            if($requested && $requested instanceof ScopedQuantityInterface) {
                /** @var ScopedQuantityInterface $requested */
                $qty -= $requested->scope()->quantityFor($sellingUnitId->productId(), $collection);
            }

            return new PropertyTotalQuantity($qty,
                $product->measureUnit(),
                $this->stockCheck,
                $this->attributeStockCheck,
                $product->getPropertiesQuantityCheck(),
                $collection);
        } catch (ProductDoesntUsePropertyQuantityException $e) {
            return null;
        }
    }

    /**
     * @param SellingUnitId $id
     * @return ModifierIdentifierCollectionInterface
     */
    protected function createModifierIdCollection(SellingUnitId $id): ModifierIdentifierCollectionInterface
    {
        $propertyIdList = [];
        foreach ($id->modifiers() as $modifierId) {
            if ($modifierId instanceof PropertyModifierIdentifier) {
                $propertyIdList[] = $modifierId;
            }
        }
        return new ModifierIdentifierCollection($propertyIdList);
    }
}